import json

from langchain.schema import Document
from transwarp_hippo_langchain.embedding_hub_langchain import TranswarpEmbeddingHub
from transwarp_hippo_langchain.hippo_langchain import Hippo

import argparse
import requests

import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

# 创建一个解析器
parser = argparse.ArgumentParser(description="Your script description")
# 添加你需要的参数
parser.add_argument("--ip", help="IP address")
parser.add_argument("--port", type=int, help="Port number")
parser.add_argument("--table_name", required=True, help="Table name")
parser.add_argument("--database_name", required=True, help="Database name")
# 解析参数
args = parser.parse_args()
# 检查是否提供了ip和port
if args.ip is None or args.port is None:
    parser.error("Please provide the connection details for the Hippo")
# 检查是否提供了table_name
if args.table_name is None:
    parser.error("Please provide the table name")
# 检查是否提供了database_name
if args.database_name is None:
    parser.error("Please provide the database name")


def chatglm_request(query: str):
    headers = {
        'Authorization': 'Bearer eyJhbGciOiJIUzUxMiIsInR5cCI6IkpXVCJ9.eyJleHAiOjQ4MDk5ODIwNzUsImlhdCI6MTY1NjM4MjA3NSwicm9sZXMiOiJbXCJTT1BIT05fQURNSU5cIixcImFkbWluXCIsXCJST0xFX0FETUlOXCJdIiwic2NvcGUiOiJleHRlcm5hbCIsInVzZXJuYW1lIjoidGhpbmdlciJ9.NAl4iMaAwMiiFRdc35pRRjdyZYzFuJvJMd9lnvz8M3xDzj8ThtokDXqFVE6SXG0vwe20UMwJtpfZJEhvfYgjHQ',
        'Content-Type': 'application/json'
    }

    data = {
        "body": "{\"JSON\":\"{\\\"query\\\":\\\"你能做什么？\\\",\\\"history\\\":[]}\"}"
    }

    # 解析外层 JSON
    body = json.loads(data['body'])

    # 解析 body 的值
    json_str = json.loads(body['JSON'])

    # 替换 query 的值
    json_str['query'] = query

    # 将修改后的字典转回 JSON 字符串
    body['JSON'] = json.dumps(json_str)

    # 将修改后的 body 转回 JSON 字符串
    data['body'] = json.dumps(body)

    response = requests.post(
        'https://172.18.192.207:30443/gateway/mw/api/v1/mwh/svcmgr/services/MWH-DEPLOYMENT-cioj641165itbd18mhqg/infer?is_stream_infer=true',
        headers=headers,
        data=json.dumps(data),
        verify=False
    )

    result = response.text
    lines = result.split('\n')
    result = ''.join(line.replace('data:', '').strip() for line in lines)
    return result

# 创建embedding model
embeddings = TranswarpEmbeddingHub(model="TranswarpVectorPulse")

# 链接向量数据库
connection = {
    "host": args.ip,
    "port": args.port,
}

vector_store = Hippo.from_documents(
    [Document(page_content="", metadata={})],
    embedding=embeddings,
    table_name=args.table_name,
    database_name=args.database_name,
    connection_args=connection
)

# 进行会话
count = 0
while True:

    if count == 0:
        print(chatglm_request("你好"))
        count += 1

    query = input("\nEnter a query: ")
    print(f"question:{query}")
    if query == "exit":
        break
    if query.strip() == "":
        continue

    else:
        # 创建检索器 （获取最相似的一篇文章）
        res = vector_store.similarity_search(query, 1)
        # 从知识库中检索 获取相似内容
        content_list = [item.page_content for item in res]

        # 接下来，用 join() 函数将这些内容拼接起来
        text = ''.join(content_list)

        # print(f"检索到相似的文章内容为："
        #       f"{text}")


        prompt = f"""使用下面的 Article：下的内容来回答我的问题. 我的问题位于Question:后面。If the answer cannot be found, write "I don't know."

             Article:
             \"\"\"
             {text}
             \"\"\"

             Question: {query}"""
        result = chatglm_request(prompt)
        print(f"AI的回答：{result}")
